<?php

namespace Drupal\views_filters_summary\Plugin\views\area;

use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeBundleInfoInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\views\Plugin\views\area\AreaPluginBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Views filters summary.
 *
 * @ingroup views_area_handlers
 *
 * @ViewsArea("views_filters_summary");
 */
class ViewsFiltersSummary extends AreaPluginBase {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The bundle info service.
   *
   * @var \Drupal\Core\Entity\EntityTypeBundleInfoInterface
   */
  protected $bundleInfoService;

  /**
   * Constructs a ViewsFiltersSummary object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Entity\EntityTypeBundleInfoInterface $bundle_info_service
   *   The bundle info service.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, EntityTypeManagerInterface $entity_type_manager, EntityTypeBundleInfoInterface $bundle_info_service) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);

    $this->entityTypeManager = $entity_type_manager;
    $this->bundleInfoService = $bundle_info_service;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('entity_type.bundle.info')
    );
  }


  /**
   * {@inheritdoc}
   */
  protected function defineOptions() {
    $options = parent::defineOptions();
    $options['filters'] = ['default' => NULL];
    $options['show_labels'] = ['default' => NULL];
    $options['show_operators'] = ['default' => NULL];
    return $options;
  }

  /**
   * {@inheritdoc}
   */
  public function buildOptionsForm(&$form, FormStateInterface $form_state) {
    parent::buildOptionsForm($form, $form_state);

    $this->view->initHandlers();

    $filter_options = array();
    foreach ($this->view->filter as $id => $handler) {
      if ($handler->isExposed()) {
        $filter_options[$id] = $handler->adminLabel();
      }
    }
    $form['filters'] = [
      '#type' => 'select',
      '#title' => $this->t('Filters'),
      '#multiple' => TRUE,
      '#options' => $filter_options,
      '#default_value' => $this->options['filters'],
      '#description' => $this->t('Choose the filters, if none are selected, all will be used.'),
    ];

    $form['show_labels'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Display labels'),
      '#default_value' => $this->options['show_labels'],
      '#description' => $this->t('If checked, the labels will be displayed, otherwise just the value is printed.'),
    ];

    $form['show_operators'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Display operators'),
      '#default_value' => $this->options['show_operators'],
      '#description' => $this->t('If checked, the operators will be displayed, rather than just foo : bar.'),
      // TODO
      '#access' => FALSE,
    ];

  }

  /**
   * {@inheritdoc}
   */
  public function render($empty = FALSE) {
    $summary = [];
    $show_label = $this->options['show_labels'];
    $selected_filters = !empty($this->options['filters']) ? $this->options['filters'] : [];

    $current_url = Url::fromRoute('<current>');
    $request = \Drupal::requestStack()->getMasterRequest();
    $current_url->setOption('query', $request->query->all());

    foreach ($this->view->filter as $id => $filter) {

      // Ignore non exposed filters, not selected filters or empty values.
      if (!$filter->isExposed() || ($selected_filters && !isset($selected_filters[$id])) || empty($filter->value)) {
        continue;
      }

      $url = clone $current_url;
      $options = $url->getOption('query');
      // TODO handle multiple values
      foreach ($options as $k => $v) {
        if ($k == $id) {
          unset($options[$k]);
        }
      }
      $url->setOption('query', $options);

      // Get the best label possible.
      $label = $filter->definition['title short'] ?? $filter->definition['title'];
      if (isset($filter->options) && isset($filter->options['expose'])) {
        if (!empty($filter->options['expose']['label'])) {
          $label = $filter->options['expose']['label'];
        }
      }

      $value = $filter->value[0];
      switch ($filter->definition['id']) {
        case 'search_api_term':
        case 'taxonomy_index_tid':
        case 'taxonomy_index_tid_depth':
          $term = $this->entityTypeManager->getStorage('taxonomy_term')->load($value);
          if ($term) {
            $value = $term->label();
          }
          break;
        case 'bundle':
          $types = $this->bundleInfoService->getBundleInfo($filter->definition['entity_type']);
          if (isset($types[$value])) {
            $value = $types[$value]['label'];
          }
          break;
      }

      $summary[] = [
        'id' => $id,
        'label' => $label,
        'show_label' => $show_label,
        'value' => $value,
        'url' => $url->toString(),
      ];

    }

    return [
      '#theme' => 'views_filters_summary',
      '#summary' => $summary,
      '#attached' => ['library' => ['views_filters_summary/output']],
    ];
  }

}
